package com.mosty.common.base.domain;

import com.github.pagehelper.PageInfo;
import com.mosty.common.base.domain.page.PageUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.time.DateUtils;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;

import java.beans.PropertyEditorSupport;
import java.text.ParseException;
import java.util.Date;
import java.util.List;

/**
 * web层通用数据处理
 *
 * @author ruoyi
 */
@Slf4j
public class BaseController {

    /**
     * 将前台传递过来的日期格式的字符串，自动转化为Date类型
     */
    @InitBinder
    public void initBinder(WebDataBinder binder) {
        // Date 类型转换
        binder.registerCustomEditor(Date.class, new PropertyEditorSupport() {
            @Override
            public void setAsText(String text) {
                try {
                    setValue(DateUtils.parseDate(text));
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
        });
    }

    /**
     * 设置请求分页数据
     */
    protected void startPage() {
        PageUtils.startPage();
    }
//
//    /**
//     * 设置请求排序数据
//     */
//    protected void startOrderBy()
//    {
//        PageDomain pageDomain = TableSupport.buildPageRequest();
//        if (StringUtils.isNotEmpty(pageDomain.getOrderBy()))
//        {
//            String orderBy = SqlUtil.escapeOrderBySql(pageDomain.getOrderBy());
//            PageHelper.orderBy(orderBy);
//        }
//    }
//
//    /**
//     * 获取request
//     */
//    public HttpServletRequest getRequest()
//    {
//        return ServletUtils.getRequest();
//    }
//
//    /**
//     * 获取response
//     */
//    public HttpServletResponse getResponse()
//    {
//        return ServletUtils.getResponse();
//    }
//
//    /**
//     * 获取session
//     */
//    public HttpSession getSession()
//    {
//        return getRequest().getSession();
//    }
//

    /**
     * 响应请求分页数据
     */
    @SuppressWarnings({"rawtypes", "unchecked"})
    protected TableDataInfo getDataTable(List<?> list) {
        TableDataInfo rspData = new TableDataInfo();
        rspData.setCode(0);
        rspData.setRows(list);
        rspData.setTotal(new PageInfo(list).getTotal());
        return rspData;
    }
//
//    /**
//     * 响应返回结果
//     *
//     * @param rows 影响行数
//     * @return 操作结果
//     */
//    protected AjaxResult toAjax(int rows)
//    {
//        return rows > 0 ? success() : error();
//    }
//
//    /**
//     * 响应返回结果
//     *
//     * @param result 结果
//     * @return 操作结果
//     */
//    protected AjaxResult toAjax(boolean result)
//    {
//        return result ? success() : error();
//    }
//
//    /**
//     * 返回成功
//     */
//    public AjaxResult success()
//    {
//        return AjaxResult.success();
//    }
//
//    /**
//     * 返回失败消息
//     */
//    public AjaxResult error()
//    {
//        return AjaxResult.error();
//    }
//
//    /**
//     * 返回成功消息
//     */
//    public AjaxResult success(String message)
//    {
//        return AjaxResult.success(message);
//    }
//
//    /**
//     * 返回成功数据
//     */
//    public static AjaxResult success(Object data)
//    {
//        return AjaxResult.success("操作成功", data);
//    }
//
//    /**
//     * 返回失败消息
//     */
//    public AjaxResult error(String message)
//    {
//        return AjaxResult.error(message);
//    }
//
//    /**
//     * 返回错误码消息
//     */
//    public AjaxResult error(Type type, String message)
//    {
//        return new AjaxResult(type, message);
//    }
//
//    /**
//     * 页面跳转
//     */
//    public String redirect(String url)
//    {
//        return StringUtils.format("redirect:{}", url);
//    }
//
//    /**
//     * 获取用户缓存信息
//     */
//    public SysUser getSysUser()
//    {
//        return ShiroUtils.getSysUser();
//    }
//
//    /**
//     * 设置用户缓存信息
//     */
//    public void setSysUser(SysUser user)
//    {
//        ShiroUtils.setSysUser(user);
//    }
//
//    /**
//     * 获取登录用户id
//     */
//    public Long getUserId()
//    {
//        return getSysUser().getUserId();
//    }
//
//    /**
//     * 获取登录用户名
//     */
//    public String getLoginName()
//    {
//        return getSysUser().getLoginName();
//    }
}
