package com.mosty.common.config.Excel;

import java.util.Collection;
import java.util.List;

import com.mosty.common.base.constant.Constants;
import com.mosty.common.base.entity.log.SpringUtils;
import com.mosty.common.base.entity.log.StringUtils;
import com.mosty.common.config.entity.SysDictItem;
import com.mosty.common.redis.service.RedisService;

/**
 * 字典工具类
 *
 * @author ruoyi
 */
public class DictUtils
{
    /**
     * 设置字典缓存
     *
     * @param key 参数键
     * @param dictDatas 字典数据列表
     */
    public static void setDictCache(String key, List<SysDictItem> dictDatas)
    {
        SpringUtils.getBean(RedisService.class).setCacheObject(getCacheKey(key), dictDatas);
    }

    /**
     * 获取字典缓存
     *
     * @param key 参数键
     * @return dictDatas 字典数据列表
     */
    public static List<SysDictItem> getDictCache(String key)
    {
        Object cacheObj = SpringUtils.getBean(RedisService.class).getCacheObject(getCacheKey(key));
        if (StringUtils.isNotNull(cacheObj))
        {
            List<SysDictItem> dictItems = StringUtils.cast(cacheObj);
            return dictItems;
        }
        return null;
    }

    /**
     * 清空字典缓存
     */
    public static void clearDictCache()
    {
        Collection<String> keys = SpringUtils.getBean(RedisService.class).keys(Constants.SYS_DICT_KEY + "*");
        SpringUtils.getBean(RedisService.class).deleteObject(keys);
    }

    /**
     * 设置cache key
     *
     * @param configKey 参数键
     * @return 缓存键key
     */
    public static String getCacheKey(String configKey)
    {
        return Constants.SYS_DICT_KEY + configKey;
    }

    /**
     * 根据指定的type ，value 获取 label
     * @param value
     * @param type
     * @param defaultLabel 默认label
     * @return
     */
    public static String getDictLabelByTypeAndValue(String value, String type, String defaultLabel){
        if (StringUtils.isNotBlank(type) && StringUtils.isNotBlank(value)){
            for (SysDictItem dict : getDictCache(type)){
                if (type.equals(dict.getZdbh()) && value.equals(dict.getDm())){
                    return dict.getZdmc();
                }
            }
        }
        return defaultLabel;
    }

    /**
     * 根据type,label 获取value
     *
     * @param label
     * @param type
     * @param defaultValue
     * @return
     */
    public static String getDictValueByTypeAndLabel(String label,String type,String defaultValue) {
        if (StringUtils.isNotBlank(type) && StringUtils.isNotBlank(label)){
            for (SysDictItem dict : getDictCache(type)){
                if(type.equals(dict.getZdbh())  && label.equals(dict.getZdmc())) {
                    return dict.getDm();
                }
            }
        }
        return defaultValue;
    }

    /**
     * 根据 type 将 lable组成 string数组
     * @param type
     * @return
     */
    public static String[] getLabelArr(String type) {
        String[] arr0 = new String[0];
        if (StringUtils.isNotBlank(type)) {
            List<SysDictItem> dictList = getDictCache(type);
            String[] strArr = new String[dictList.size()];
            for (int i = 0 ; i < dictList.size() ; i ++) {
                strArr[i] = dictList.get(i).getZdmc();
            }
            return strArr;
        }
        return arr0;
    }

    /**
     * 删除指定字典缓存
     *
     * @param key 字典键
     */
    public static void removeDictCache(String key)
    {
        SpringUtils.getBean(RedisService.class).deleteObject(getCacheKey(key));
    }
}
