package com.mosty.common.generator.controller;

import com.mosty.common.base.domain.BaseController;
import com.mosty.common.base.domain.ResponseResult;
import com.mosty.common.base.domain.TableDataInfo;
import com.mosty.common.base.entity.log.BusinessType;
import com.mosty.common.base.entity.log.Log;
import com.mosty.common.generator.domain.GenTable;
import com.mosty.common.generator.domain.GenTableColumn;
import com.mosty.common.generator.service.IGenTableColumnService;
import com.mosty.common.generator.service.IGenTableService;
import com.mosty.common.token.JwtSysUser;
import com.mosty.operation.log.util.Convert;
import io.swagger.annotations.Api;
import org.apache.commons.io.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 代码生成 操作处理
 *
 * @author ruoyi
 */
@RequestMapping("/gen")
@RestController
@Api(tags = "代码生成")
public class GenController extends BaseController {
    @Autowired
    private IGenTableService genTableService;

    @Autowired
    private IGenTableColumnService genTableColumnService;

    /**
     * 查询代码生成列表
     */
//    @RequiresPermissions("tool:gen:list")
    @GetMapping("/list")
    public ResponseResult<TableDataInfo> genList(GenTable genTable) {
        startPage();
        List<GenTable> list = genTableService.selectGenTableList(genTable);
        return ResponseResult.success(getDataTable(list));
    }

    /**
     * 修改代码生成业务
     */
//    @RequiresPermissions("tool:gen:query")
    @GetMapping(value = "/{talbleId}")
    public ResponseResult<Map<String, Object>> getInfo(@PathVariable Long tableId) {
        GenTable table = genTableService.selectGenTableById(tableId);
        List<GenTable> tables = genTableService.selectGenTableAll();
        List<GenTableColumn> list = genTableColumnService.selectGenTableColumnListByTableId(tableId);
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("info", table);
        map.put("rows", list);
        map.put("tables", tables);
        return ResponseResult.success(map);
    }

    /**
     * 查询数据库列表
     */
//    @RequiresPermissions("tool:gen:list")
    @GetMapping("/db/list")
    public ResponseResult<TableDataInfo> dataList(GenTable genTable) {
        startPage();
        List<GenTable> list = genTableService.selectDbTableList(genTable);
        return ResponseResult.success(getDataTable(list));
    }

    /**
     * 查询数据表字段列表
     */
    @GetMapping(value = "/column/{talbleId}")
    public ResponseResult<TableDataInfo> columnList(Long tableId) {
        TableDataInfo dataInfo = new TableDataInfo();
        List<GenTableColumn> list = genTableColumnService.selectGenTableColumnListByTableId(tableId);
        dataInfo.setRows(list);
        dataInfo.setTotal(list.size());
        return ResponseResult.success(dataInfo);
    }

    /**
     * 导入表结构（保存）
     */
//    @RequiresPermissions("tool:gen:import")
    @Log(title = "代码生成", businessType = BusinessType.IMPORT)
    @PostMapping("/importTable")
    @JwtSysUser
    public ResponseResult<Boolean> importTableSave(String tables) {
        String[] tableNames = Convert.toStrArray(tables);
        // 查询表信息
        List<GenTable> tableList = genTableService.selectDbTableListByNames(tableNames);
        genTableService.importGenTable(tableList);
        return ResponseResult.success(Boolean.TRUE);
    }

    /**
     * 修改保存代码生成业务
     */
//    @RequiresPermissions("tool:gen:edit")
    @Log(title = "代码生成", businessType = BusinessType.UPDATE)
    @PutMapping
    public ResponseResult<Boolean> editSave(@Validated @RequestBody GenTable genTable) {
        genTableService.validateEdit(genTable);
        genTableService.updateGenTable(genTable);
        return ResponseResult.success(Boolean.TRUE);
    }

    /**
     * 删除代码生成
     */
//    @RequiresPermissions("tool:gen:remove")
    @Log(title = "代码生成", businessType = BusinessType.DELETE)
    @DeleteMapping("/{tableIds}")
    public ResponseResult<Boolean> remove(@PathVariable Long[] tableIds) {
        genTableService.deleteGenTableByIds(tableIds);
        return ResponseResult.success(Boolean.TRUE);
    }

    /**
     * 预览代码
     */
//    @RequiresPermissions("tool:gen:preview")
    @GetMapping("/preview/{tableId}")
    public ResponseResult<Map<String, String>> preview(@PathVariable("tableId") Long tableId) throws IOException {
        Map<String, String> dataMap = genTableService.previewCode(tableId);
        return ResponseResult.success(dataMap);
    }

    /**
     * 生成代码（下载方式）
     */
//    @RequiresPermissions("tool:gen:code")
    @Log(title = "代码生成", businessType = BusinessType.GENCODE)
    @GetMapping("/download/{tableName}")
    public void download(HttpServletResponse response, @PathVariable("tableName") String tableName) throws IOException {
        byte[] data = genTableService.downloadCode(tableName);
        genCode(response, data);
    }

    /**
     * 生成代码（自定义路径）
     */
//    @RequiresPermissions("tool:gen:code")
    @Log(title = "代码生成", businessType = BusinessType.GENCODE)
    @GetMapping("/genCode/{tableName}")
    public ResponseResult<Boolean> genCode(@PathVariable("tableName") String tableName) {
        genTableService.generatorCode(tableName);
        return ResponseResult.success(Boolean.TRUE);
    }

    /**
     * 同步数据库
     */
//    @RequiresPermissions("tool:gen:edit")
    @Log(title = "代码生成", businessType = BusinessType.UPDATE)
    @GetMapping("/synchDb/{tableName}")
    public ResponseResult synchDb(@PathVariable("tableName") String tableName) {
        genTableService.synchDb(tableName);
        return ResponseResult.success();
    }

    /**
     * 批量生成代码
     */
//    @RequiresPermissions("tool:gen:code")
    @Log(title = "代码生成", businessType = BusinessType.GENCODE)
    @GetMapping("/batchGenCode")
    public void batchGenCode(HttpServletResponse response, String tables) throws IOException {
        String[] tableNames = Convert.toStrArray(tables);
        byte[] data = genTableService.downloadCode(tableNames);
        genCode(response, data);
    }

    /**
     * 生成zip文件
     */
    private void genCode(HttpServletResponse response, byte[] data) throws IOException {
        response.reset();
        response.setHeader("Content-Disposition", "attachment; filename=\"ruoyi.zip\"");
        response.addHeader("Content-Length", "" + data.length);
        response.setContentType("application/octet-stream; charset=UTF-8");
        IOUtils.write(data, response.getOutputStream());
    }
}
